@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright © Spyder Project Contributors
@@ -21,16 +21,15 @@ import sys
 import time
 
 # Third party imports
-from qtpy import PYQT5
 from qtpy.QtCore import Qt, QTimer
-from qtpy.QtGui import QColor, QPixmap, QIcon
+from qtpy.QtGui import QColor, QPixmap
 from qtpy.QtWidgets import QApplication, QMessageBox, QSplashScreen, QWidget
 
 # Local imports
 from spyder.config.base import _, get_image_path
 from spyder.py3compat import to_text_string
+from spyder.utils import icon_manager as ima
 from spyder.utils.qthelpers import qapplication
-from spyder.config.main import CONF
 
 
 PY2 = sys.version[0] == '2'
@@ -154,32 +153,15 @@ class Restarter(QWidget):
 
 
 def main():
-    #==========================================================================
-    # Proper high DPI scaling is available in Qt >= 5.6.0. This attibute must
-    # be set before creating the application.
-    #==========================================================================
-    if CONF.get('main', 'high_dpi_custom_scale_factor'):
-        factors = str(CONF.get('main', 'high_dpi_custom_scale_factors'))
-        f = list(filter(None, factors.split(';')))
-        if len(f) == 1:
-            os.environ['QT_SCALE_FACTOR'] = f[0]
-        else:
-            os.environ['QT_SCREEN_SCALE_FACTORS'] = factors
-    else:
-        os.environ['QT_SCALE_FACTOR'] = ''
-        os.environ['QT_SCREEN_SCALE_FACTORS'] = ''
-
     # Splash screen
     # -------------------------------------------------------------------------
     # Start Qt Splash to inform the user of the current status
     app = qapplication()
     restarter = Restarter()
-
-    if PYQT5:
-        APP_ICON = QIcon(get_image_path("spyder.svg"))
-    else:
-        APP_ICON = QIcon(get_image_path("spyder.png"))
-    app.setWindowIcon(APP_ICON)
+    resample = not IS_WINDOWS
+    # Resampling SVG icon only on non-Windows platforms (see Issue 1314):
+    icon = ima.icon('spyder', resample=resample)
+    app.setWindowIcon(icon)
     restarter.set_splash_message(_('Closing Spyder'))
 
     # Get variables
