@@ -1,13 +1,11 @@
 # -*- coding: utf-8 -*-
-# -----------------------------------------------------------------------------
-# Copyright © Spyder Project Contributors
 #
+# Copyright © Spyder Project Contributors
 # Licensed under the terms of the MIT License
 # (see spyder/__init__.py for details)
-# ----------------------------------------------------------------------------
 
 """
-Collections (i.e. dictionary, list and tuple) editor widget and dialog.
+Collections (i.e. dictionary, list and tuple) editor widget and dialog
 """
 
 #TODO: Multiple selection: open as many editors (array/dict/...) as necessary,
@@ -35,14 +33,14 @@ from qtpy.QtWidgets import (QAbstractItemDelegate, QApplication, QDateEdit,
                             QInputDialog, QItemDelegate, QLineEdit, QMenu,
                             QMessageBox, QTableView, QVBoxLayout, QWidget)
 
-# Local imports
+# Local import
 from spyder.config.base import _
 from spyder.config.fonts import DEFAULT_SMALL_DELTA
 from spyder.config.gui import get_font
 from spyder.py3compat import (io, is_binary_string, is_text_string,
-                              PY3, to_text_string)
+                              getcwd, PY3, to_text_string)
 from spyder.utils import icon_manager as ima
-from spyder.utils.misc import fix_reference_name, getcwd_or_home
+from spyder.utils.misc import fix_reference_name
 from spyder.utils.qthelpers import (add_actions, create_action,
                                     mimedata2url)
 from spyder.widgets.variableexplorer.importwizard import ImportWizard
@@ -62,7 +60,6 @@ if DataFrame is not FakeObject:
 
 
 LARGE_NROWS = 100
-ROWS_TO_LOAD = 50
 
 
 class ProxyObject(object):
@@ -77,37 +74,20 @@ class ProxyObject(object):
         return len(get_object_attrs(self.__obj__))
 
     def __getitem__(self, key):
-        """Get the attribute corresponding to the given key."""
-        # Catch NotImplementedError to fix #6284 in pandas MultiIndex
-        # due to NA checking not being supported on a multiindex.
-        # Catch AttributeError to fix #5642 in certain special classes like xml
-        # when this method is called on certain attributes.
-        # Catch TypeError to prevent fatal Python crash to desktop after
-        # modifying certain pandas objects. Fix issue #6727 .
-        # Catch ValueError to allow viewing and editing of pandas offsets.
-        # Fix issue #6728 .
-        try:
-            attribute_toreturn = getattr(self.__obj__, key)
-        except (NotImplementedError, AttributeError, TypeError, ValueError):
-            attribute_toreturn = None
-        return attribute_toreturn
+        """Get attribute corresponding to key."""
+        return getattr(self.__obj__, key)
 
     def __setitem__(self, key, value):
         """Set attribute corresponding to key with value."""
-        # Catch AttributeError to gracefully handle inability to set an
-        # attribute due to it not being writeable or set-table.
-        # Fix issue #6728 . Also, catch NotImplementedError for safety.
         try:
             setattr(self.__obj__, key, value)
-        except (TypeError, AttributeError, NotImplementedError):
+        except TypeError:
             pass
-        except Exception as e:
-            if "cannot set values for" not in str(e):
-                raise
 
 
 class ReadOnlyCollectionsModel(QAbstractTableModel):
     """CollectionsEditor Read-Only Table Model"""
+    ROWS_TO_LOAD = 50
 
     def __init__(self, parent, data, title="", names=False,
                  minmax=False, dataframe_format=None, remote=False):
@@ -172,7 +152,7 @@ class ReadOnlyCollectionsModel(QAbstractTableModel):
 
         self.total_rows = len(self.keys)
         if self.total_rows > LARGE_NROWS:
-            self.rows_loaded = ROWS_TO_LOAD
+            self.rows_loaded = self.ROWS_TO_LOAD
         else:
             self.rows_loaded = self.total_rows
 
@@ -218,22 +198,27 @@ class ReadOnlyCollectionsModel(QAbstractTableModel):
             except:
                 pass
         elif column == 1:
-            self.keys[:self.rows_loaded] = sort_against(self.keys, self.types,
-                                                        reverse)
+            self.keys = sort_against(self.keys, self.types, reverse)
             self.sizes = sort_against(self.sizes, self.types, reverse)
             try:
                 self.types.sort(reverse=reverse)
             except:
                 pass
         elif column == 2:
-            self.keys[:self.rows_loaded] = sort_against(self.keys, self.sizes,
-                                                        reverse)
+            self.keys = sort_against(self.keys, self.sizes, reverse)
             self.types = sort_against(self.types, self.sizes, reverse)
             try:
                 self.sizes.sort(reverse=reverse)
             except:
                 pass
         elif column == 3:
+            self.keys = sort_against(self.keys, self.sizes, reverse)
+            self.types = sort_against(self.types, self.sizes, reverse)
+            try:
+                self.sizes.sort(reverse=reverse)
+            except:
+                pass
+        elif column == 4:
             values = [self._data[key] for key in self.keys]
             self.keys = sort_against(self.keys, values, reverse)
             self.sizes = sort_against(self.sizes, values, reverse)
@@ -260,7 +245,7 @@ class ReadOnlyCollectionsModel(QAbstractTableModel):
  
     def fetchMore(self, index=QModelIndex()):
         reminder = self.total_rows - self.rows_loaded
-        items_to_fetch = min(reminder, ROWS_TO_LOAD)
+        items_to_fetch = min(reminder, self.ROWS_TO_LOAD)
         self.set_size_and_type(self.rows_loaded,
                                self.rows_loaded + items_to_fetch)
         self.beginInsertRows(QModelIndex(), self.rows_loaded,
@@ -271,7 +256,7 @@ class ReadOnlyCollectionsModel(QAbstractTableModel):
     def get_index_from_key(self, key):
         try:
             return self.createIndex(self.keys.index(key), 0)
-        except (RuntimeError, ValueError):
+        except ValueError:
             return QModelIndex()
     
     def get_key(self, index):
@@ -453,30 +438,30 @@ class CollectionsDelegate(QItemDelegate):
                                    ) % to_text_string(msg))
             return
         key = index.model().get_key(index)
-        readonly = (isinstance(value, tuple) or self.parent().readonly
-                    or not is_known_type(value))
-        # CollectionsEditor for a list, tuple, dict, etc.
+        readonly = isinstance(value, tuple) or self.parent().readonly \
+                   or not is_known_type(value)
+        #---editor = CollectionsEditor
         if isinstance(value, (list, tuple, dict)):
-            editor = CollectionsEditor(parent=parent)
+            editor = CollectionsEditor()
             editor.setup(value, key, icon=self.parent().windowIcon(),
                          readonly=readonly)
             self.create_dialog(editor, dict(model=index.model(), editor=editor,
                                             key=key, readonly=readonly))
             return None
-        # ArrayEditor for a Numpy array
+        #---editor = ArrayEditor
         elif isinstance(value, (ndarray, MaskedArray)) \
           and ndarray is not FakeObject:
-            editor = ArrayEditor(parent=parent)
+            editor = ArrayEditor(parent)
             if not editor.setup_and_check(value, title=key, readonly=readonly):
                 return
             self.create_dialog(editor, dict(model=index.model(), editor=editor,
                                             key=key, readonly=readonly))
             return None
-        # ArrayEditor for an images
+        #---showing image
         elif isinstance(value, Image) and ndarray is not FakeObject \
           and Image is not FakeObject:
             arr = array(value)
-            editor = ArrayEditor(parent=parent)
+            editor = ArrayEditor(parent)
             if not editor.setup_and_check(arr, title=key, readonly=readonly):
                 return
             conv_func = lambda arr: Image.fromarray(arr, mode=value.mode)
@@ -484,10 +469,10 @@ class CollectionsDelegate(QItemDelegate):
                                             key=key, readonly=readonly,
                                             conv=conv_func))
             return None
-        # DataFrameEditor for a pandas dataframe, series or index
+        #--editor = DataFrameEditor
         elif isinstance(value, (DataFrame, DatetimeIndex, Series)) \
           and DataFrame is not FakeObject:
-            editor = DataFrameEditor(parent=parent)
+            editor = DataFrameEditor()
             if not editor.setup_and_check(value, title=key):
                 return
             editor.dataModel.set_format(index.model().dataframe_format)
@@ -495,51 +480,44 @@ class CollectionsDelegate(QItemDelegate):
             self.create_dialog(editor, dict(model=index.model(), editor=editor,
                                             key=key, readonly=readonly))
             return None
-        # QDateEdit and QDateTimeEdit for a dates or datetime respectively
+        #---editor = QDateTimeEdit
+        elif isinstance(value, datetime.datetime):
+            editor = QDateTimeEdit(value, parent)
+            editor.setCalendarPopup(True)
+            editor.setFont(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
+            return editor
+        #---editor = QDateEdit
         elif isinstance(value, datetime.date):
-            if readonly:
-                return None
-            else:
-                if isinstance(value, datetime.datetime):
-                    editor = QDateTimeEdit(value, parent=parent)
-                else:
-                    editor = QDateEdit(value, parent=parent)
-                editor.setCalendarPopup(True)
-                editor.setFont(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
-                return editor
-        # TextEditor for a long string
+            editor = QDateEdit(value, parent)
+            editor.setCalendarPopup(True)
+            editor.setFont(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
+            return editor
+        #---editor = TextEditor
         elif is_text_string(value) and len(value) > 40:
-            te = TextEditor(None, parent=parent)
-            if te.setup_and_check(value):
-                editor = TextEditor(value, key,
-                                    readonly=readonly, parent=parent)
-                self.create_dialog(editor, dict(model=index.model(),
-                                                editor=editor, key=key,
-                                                readonly=readonly))
+            editor = TextEditor(value, key)
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly))
             return None
-        # QLineEdit for an individual value (int, float, short string, etc)
+        #---editor = QLineEdit
         elif is_editable_type(value):
-            if readonly:
-                return None
-            else:
-                editor = QLineEdit(parent=parent)
-                editor.setFont(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
-                editor.setAlignment(Qt.AlignLeft)
-                # This is making Spyder crash because the QLineEdit that it's
-                # been modified is removed and a new one is created after
-                # evaluation. So the object on which this method is trying to
-                # act doesn't exist anymore.
-                # editor.returnPressed.connect(self.commitAndCloseEditor)
-                return editor
-        # CollectionsEditor for an arbitrary Python object
+            editor = QLineEdit(parent)
+            editor.setFont(get_font(font_size_delta=DEFAULT_SMALL_DELTA))
+            editor.setAlignment(Qt.AlignLeft)
+            # This is making Spyder crash because the QLineEdit that it's
+            # been modified is removed and a new one is created after
+            # evaluation. So the object on which this method is trying to
+            # act doesn't exist anymore.
+            # editor.returnPressed.connect(self.commitAndCloseEditor)
+            return editor
+        #---editor = CollectionsEditor for an arbitrary object
         else:
-            editor = CollectionsEditor(parent=parent)
+            editor = CollectionsEditor()
             editor.setup(value, key, icon=self.parent().windowIcon(),
                          readonly=readonly)
             self.create_dialog(editor, dict(model=index.model(), editor=editor,
                                             key=key, readonly=readonly))
             return None
-
+            
     def create_dialog(self, editor, data):
         self._editors[id(editor)] = data
         editor.accepted.connect(
@@ -695,7 +673,7 @@ class BaseTableView(QTableView):
                                           triggered=self.paste)
         self.copy_action = create_action(self, _("Copy"),
                                          icon=ima.icon('editcopy'),
-                                         triggered=self.copy)
+                                         triggered=self.copy)                                      
         self.edit_action = create_action(self, _("Edit"),
                                          icon=ima.icon('edit'),
                                          triggered=self.edit_item)
@@ -976,12 +954,8 @@ class BaseTableView(QTableView):
         else:
             title = _('Duplicate')
             field_text = _('Variable name:')
-        data = self.model.get_data()
-        if isinstance(data, (list, set)):
-            new_key, valid = len(data), True
-        else:
-            new_key, valid = QInputDialog.getText(self, title, field_text,
-                                                  QLineEdit.Normal, orig_key)
+        new_key, valid = QInputDialog.getText(self, title, field_text,
+                                              QLineEdit.Normal, orig_key)
         if valid and to_text_string(new_key):
             new_key = try_to_eval(to_text_string(new_key))
             if new_key == orig_key:
@@ -1030,13 +1004,13 @@ class BaseTableView(QTableView):
         try:
             import guiqwt.pyplot   #analysis:ignore
             return True
-        except:
+        except (ImportError, AssertionError):
             try:
                 if 'matplotlib' not in sys.modules:
                     import matplotlib
                     matplotlib.use("Qt4Agg")
                 return True
-            except:
+            except ImportError:
                 QMessageBox.warning(self, _("Import error"),
                                     _("Please install <b>matplotlib</b>"
                                       " or <b>guiqwt</b>."))
@@ -1076,7 +1050,7 @@ class BaseTableView(QTableView):
         """Save array"""
         title = _( "Save array")
         if self.array_filename is None:
-            self.array_filename = getcwd_or_home()
+            self.array_filename = getcwd()
         self.redirect_stdio.emit(False)
         filename, _selfilter = getsavefilename(self, title,
                                                self.array_filename,
@@ -1190,12 +1164,7 @@ class CollectionsEditorTableView(BaseTableView):
     def copy_value(self, orig_key, new_key):
         """Copy value"""
         data = self.model.get_data()
-        if isinstance(data, list):
-            data.append(data[orig_key])
-        if isinstance(data, set):
-            data.add(data[orig_key])
-        else:
-            data[new_key] = data[orig_key]
+        data[new_key] = data[orig_key]
         self.set_data(data)
     
     def new_value(self, key, value):
@@ -1276,9 +1245,6 @@ class CollectionsEditorTableView(BaseTableView):
         self.edit_action.setEnabled( condition )
         self.remove_action.setEnabled( condition )
         self.insert_action.setEnabled( not self.readonly )
-        self.duplicate_action.setEnabled(condition)
-        condition_rename = not isinstance(data, (tuple, list, set))
-        self.rename_action.setEnabled(condition_rename)
         self.refresh_plot_entries(index)
         
     def set_filter(self, dictfilter=None):
@@ -1336,31 +1302,24 @@ class CollectionsEditor(QDialog):
         else:
             # unknown object
             import copy
-            try:
-                self.data_copy = copy.deepcopy(data)
-            except NotImplementedError:
-                self.data_copy = copy.copy(data)
-            except (TypeError, AttributeError):
-                readonly = True
-                self.data_copy = data
+            self.data_copy = copy.deepcopy(data)
             datalen = len(get_object_attrs(data))
-        self.widget = CollectionsEditorWidget(self, self.data_copy,
-                                              title=title, readonly=readonly,
-                                              remote=remote)
+        self.widget = CollectionsEditorWidget(self, self.data_copy, title=title,
+                                              readonly=readonly, remote=remote)
 
         layout = QVBoxLayout()
         layout.addWidget(self.widget)
         self.setLayout(layout)
-
+        
         # Buttons configuration
         buttons = QDialogButtonBox.Ok
         if not readonly:
             buttons = buttons | QDialogButtonBox.Cancel
-        self.bbox = QDialogButtonBox(buttons)
-        self.bbox.accepted.connect(self.accept)
+        bbox = QDialogButtonBox(buttons)
+        bbox.accepted.connect(self.accept)
         if not readonly:
-            self.bbox.rejected.connect(self.reject)
-        layout.addWidget(self.bbox)
+            bbox.rejected.connect(self.reject)
+        layout.addWidget(bbox)
 
         constant = 121
         row_height = 30
@@ -1468,27 +1427,10 @@ def get_test_data():
     """Create test data."""
     import numpy as np
     from spyder.pil_patch import Image
-    image = Image.fromarray(np.random.randint(256, size=(100, 100)),
+    image = Image.fromarray(np.random.random_integers(255, size=(100, 100)),
                             mode='P')
     testdict = {'d': 1, 'a': np.random.rand(10, 10), 'b': [1, 2]}
     testdate = datetime.date(1945, 5, 8)
-    test_timedelta = datetime.timedelta(days=-1, minutes=42, seconds=13)
-
-    try:
-        import pandas as pd
-    except (ModuleNotFoundError, ImportError):
-        test_timestamp, test_pd_td, test_dtindex, test_series, test_df = None
-    else:
-        test_timestamp = pd.Timestamp("1945-05-08T23:01:00.12345")
-        test_pd_td = pd.Timedelta(days=2193, hours=12)
-        test_dtindex = pd.DatetimeIndex(start="1939-09-01T",
-                                              end="1939-10-06",
-                                              freq="12H")
-        test_series = pd.Series({"series_name": [0, 1, 2, 3, 4, 5]})
-        test_df = pd.DataFrame({"string_col": ["a", "b", "c", "d"],
-                                "int_col": [0, 1, 2, 3],
-                                "float_col": [1.1, 2.2, 3.3, 4.4],
-                                "bool_col": [True, False, False, True]})
 
     class Foobar(object):
 
@@ -1499,43 +1441,28 @@ def get_test_data():
 
     foobar = Foobar()
     return {'object': foobar,
-            'module': np,
             'str': 'kjkj kj k j j kj k jkj',
             'unicode': to_text_string('éù', 'utf-8'),
             'list': [1, 3, [sorted, 5, 6], 'kjkj', None],
-            'tuple': ([1, testdate, testdict, test_timedelta], 'kjkj', None),
+            'tuple': ([1, testdate, testdict], 'kjkj', None),
             'dict': testdict,
             'float': 1.2233,
             'int': 223,
             'bool': True,
-            'array': np.random.rand(10, 10).astype(np.int64),
+            'array': np.random.rand(10, 10),
             'masked_array': np.ma.array([[1, 0], [1, 0]],
                                         mask=[[True, False], [False, False]]),
-            '1D-array': np.linspace(-10, 10).astype(np.float16),
-            '3D-array': np.random.randint(2, size=(5, 5, 5)).astype(np.bool_),
+            '1D-array': np.linspace(-10, 10),
             'empty_array': np.array([]),
             'image': image,
             'date': testdate,
-            'datetime': datetime.datetime(1945, 5, 8, 23, 1, 0, int(1.5e5)),
-            'timedelta': test_timedelta,
+            'datetime': datetime.datetime(1945, 5, 8),
             'complex': 2+1j,
             'complex64': np.complex64(2+1j),
-            'complex128': np.complex128(9j),
             'int8_scalar': np.int8(8),
             'int16_scalar': np.int16(16),
             'int32_scalar': np.int32(32),
-            'int64_scalar': np.int64(64),
-            'float16_scalar': np.float16(16),
-            'float32_scalar': np.float32(32),
-            'float64_scalar': np.float64(64),
             'bool_scalar': np.bool(8),
-            'bool__scalar': np.bool_(8),
-            'timestamp': test_timestamp,
-            'timedelta_pd': test_pd_td,
-            'datetimeindex': test_dtindex,
-            'series': test_series,
-            'ddataframe': test_df,
-            'None': None,
             'unsupported1': np.arccos,
             'unsupported2': np.cast,
             # Test for Issue #3518
@@ -1545,7 +1472,7 @@ def get_test_data():
 
 
 def editor_test():
-    """Test Collections editor."""
+    """Collections editor test"""
     from spyder.utils.qthelpers import qapplication
 
     app = qapplication()             #analysis:ignore
@@ -1556,14 +1483,14 @@ def editor_test():
 
 
 def remote_editor_test():
-    """Test remote collections editor."""
+    """Remote collections editor test"""
     from spyder.utils.qthelpers import qapplication
     app = qapplication()
 
     from spyder.plugins.variableexplorer import VariableExplorer
     from spyder.widgets.variableexplorer.utils import make_remote_view
 
-    remote = make_remote_view(get_test_data(),
+    remote = make_remote_view(get_test_data(), 
                               VariableExplorer(None).get_settings())
     dialog = CollectionsEditor()
     dialog.setup(remote, remote=True)
