@@ -14,27 +14,24 @@ Pandas DataFrame Editor Dialog
 """
 
 # Third party imports
+from pandas import DataFrame, DatetimeIndex, Series
 from qtpy import API
 from qtpy.compat import from_qvariant, to_qvariant
 from qtpy.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal, Slot
 from qtpy.QtGui import QColor, QCursor
-from qtpy.QtWidgets import (QApplication, QCheckBox, QDialog, QGridLayout,
-                            QHBoxLayout, QInputDialog, QLineEdit, QMenu,
-                            QMessageBox, QPushButton, QTableView, QHeaderView)
-
-from pandas import DataFrame, DatetimeIndex, Series
-try:
-    from pandas._libs.tslib import OutOfBoundsDatetime
-except ImportError:  # For pandas version < 0.20
-    from pandas.tslib import OutOfBoundsDatetime
+from qtpy.QtWidgets import (QApplication, QCheckBox, QDialogButtonBox, QDialog,
+                            QGridLayout, QHBoxLayout, QInputDialog, QLineEdit,
+                            QMenu, QMessageBox, QPushButton, QTableView,
+                            QHeaderView)
 import numpy as np
 
 # Local imports
 from spyder.config.base import _
 from spyder.config.fonts import DEFAULT_SMALL_DELTA
 from spyder.config.gui import get_font, config_shortcut
-from spyder.py3compat import (io, is_text_string, is_type_text_string, PY2,
-                              to_text_string)
+from spyder.py3compat import (io, is_text_string, PY2, to_text_string,
+                              TEXT_TYPES)
+from spyder.utils import encoding
 from spyder.utils import icon_manager as ima
 from spyder.utils.qthelpers import (add_actions, create_action,
                                     keybinding, qapplication)
@@ -44,10 +41,7 @@ from spyder.widgets.variableexplorer.arrayeditor import get_idx_rect
 REAL_NUMBER_TYPES = (float, int, np.int64, np.int32)
 COMPLEX_NUMBER_TYPES = (complex, np.complex64, np.complex128)
 # Used to convert bool intrance to false since bool('False') will return True
-_bool_false = ['false', 'f', '0', '0.', '0.0', ' ']
-
-# Default format for data frames with floats
-DEFAULT_FORMAT = '%.6g'
+_bool_false = ['false', '0']
 
 # Limit at which dataframe is considered so large that it is loaded on demand
 LARGE_SIZE = 5e5
@@ -89,7 +83,7 @@ class DataFrameModel(QAbstractTableModel):
     ROWS_TO_LOAD = 500
     COLS_TO_LOAD = 40
     
-    def __init__(self, dataFrame, format=DEFAULT_FORMAT, parent=None):
+    def __init__(self, dataFrame, format="%.3g", parent=None):
         QAbstractTableModel.__init__(self)
         self.dialog = parent
         self.df = dataFrame
@@ -193,26 +187,31 @@ class DataFrameModel(QAbstractTableModel):
             if section == 0:
                 return 'Index'
             elif section == 1 and PY2:
-                header = self.df_header[0]
                 # Get rid of possible BOM utf-8 data present at the
                 # beginning of a file, which gets attached to the first
                 # column header when headers are present in the first
                 # row.
                 # Fixes Issue 2514
                 try:
-                    header = to_text_string(header, encoding='utf-8-sig')
+                    header = to_text_string(self.df_header[0],
+                                            encoding='utf-8-sig')
                 except:
-                    # Don't perform any conversion on strings because it
-                    # leads to differences between the data present in
-                    # the dataframe and what is shown by Spyder
-                    if not is_type_text_string(header):
-                        header = to_text_string(header)
+                    header = to_text_string(self.df_header[0])
                 return to_qvariant(header)
-            elif is_type_text_string(self.df_header[section-1]):
-                # Don't perform any conversion on strings because it
-                # leads to differences between the data present in
-                # the dataframe and what is shown by Spyder
-                return to_qvariant(self.df_header[section-1])
+            elif isinstance(self.df_header[section-1], TEXT_TYPES):
+                # Get the proper encoding of the text in the header.
+                # Fixes Issue 3896
+                if not PY2:
+                    try:
+                        header = self.df_header[section-1].encode('utf-8')
+                        coding = 'utf-8-sig'
+                    except:
+                        header = self.df_header[section-1].encode('utf-8')
+                        coding = encoding.get_coding(header)
+                else:
+                    header = self.df_header[section-1]
+                    coding = encoding.get_coding(header)
+                return to_qvariant(to_text_string(header, encoding=coding))
             else:
                 return to_qvariant(to_text_string(self.df_header[section-1]))
         else:
@@ -255,8 +254,6 @@ class DataFrameModel(QAbstractTableModel):
         # handling, so fallback uses iloc
         try:
             value = self.df.iat[row, column]
-        except OutOfBoundsDatetime:
-            value = self.df.iloc[:, column].astype(str).iat[row]
         except:
             value = self.df.iloc[row, column]
         return value
@@ -273,32 +270,16 @@ class DataFrameModel(QAbstractTableModel):
             column = index.column()
             row = index.row()
             if column == 0:
-                df_idx = self.df_index[row]
-                if is_type_text_string(df_idx):
-                    # Don't perform any conversion on strings
-                    # because it leads to differences between
-                    # the data present in the dataframe and
-                    # what is shown by Spyder
-                    return df_idx
-                else:
-                    return to_qvariant(to_text_string(df_idx))
+                return to_qvariant(to_text_string(self.df_index[row]))
             else:
                 value = self.get_value(row, column-1)
                 if isinstance(value, float):
-                    try:
-                        return to_qvariant(self._format % value)
-                    except (ValueError, TypeError):
-                        # may happen if format = '%d' and value = NaN;
-                        # see issue 4139
-                        return to_qvariant(DEFAULT_FORMAT % value)
-                elif is_type_text_string(value):
-                    # Don't perform any conversion on strings
-                    # because it leads to differences between
-                    # the data present in the dataframe and
-                    # what is shown by Spyder
-                    return value
+                    return to_qvariant(self._format % value)
                 else:
-                    return to_qvariant(to_text_string(value))
+                    try:
+                        return to_qvariant(to_text_string(value))
+                    except UnicodeDecodeError:
+                        return to_qvariant(encoding.to_unicode(value))
         elif role == Qt.BackgroundColorRole:
             return to_qvariant(self.get_bgcolor(index))
         elif role == Qt.FontRole:
@@ -325,14 +306,6 @@ class DataFrameModel(QAbstractTableModel):
                     self.df.sort(columns=self.df.columns[column-1],
                                  ascending=ascending, inplace=True,
                                  kind='mergesort')
-                except ValueError as e:
-                    # Not possible to sort on duplicate columns #5225
-                    QMessageBox.critical(self.dialog, "Error",
-                                         "ValueError: %s" % to_text_string(e))
-                except SystemError as e:
-                    # Not possible to sort on category dtypes #5361
-                    QMessageBox.critical(self.dialog, "Error",
-                                         "SystemError: %s" % to_text_string(e))
                 self.update_df_index()
             else:
                 self.df.sort_index(inplace=True, ascending=ascending)
@@ -369,24 +342,23 @@ class DataFrameModel(QAbstractTableModel):
         else:
             val = from_qvariant(value, str)
             current_value = self.get_value(row, column-1)
-            if isinstance(current_value, (bool, np.bool_)):
+            if isinstance(current_value, bool):
                 val = bool_false_check(val)
-            supported_types = (bool, np.bool_) + REAL_NUMBER_TYPES
-            if (isinstance(current_value, supported_types) or
+            supported_types = (bool,) + REAL_NUMBER_TYPES + COMPLEX_NUMBER_TYPES
+            if (isinstance(current_value, supported_types) or 
                     is_text_string(current_value)):
                 try:
                     self.df.iloc[row, column-1] = current_value.__class__(val)
-                except (ValueError, OverflowError) as e:
+                except ValueError as e:
                     QMessageBox.critical(self.dialog, "Error",
-                                         str(type(e).__name__) + ": " + str(e))
+                                         "Value error: %s" % str(e))
                     return False
             else:
                 QMessageBox.critical(self.dialog, "Error",
-                                     "Editing dtype {0!s} not yet supported."
-                                     .format(type(current_value).__name__))
+                                     "The type of the cell is not a supported "
+                                     "type")
                 return False
         self.max_min_col_update()
-        self.dataChanged.emit(index, index)
         return True
 
     def get_data(self):
@@ -446,7 +418,7 @@ class DataFrameModel(QAbstractTableModel):
 class FrozenTableView(QTableView):
     """This class implements a table with its first column frozen
     For more information please see:
-    https://doc.qt.io/qt-5/qtwidgets-itemviews-frozencolumn-example.html"""
+    http://doc.qt.io/qt-5/qtwidgets-itemviews-frozencolumn-example.html"""
     def __init__(self, parent):
         """Constructor."""
         QTableView.__init__(self, parent)
@@ -696,7 +668,6 @@ class DataFrameEditor(QDialog):
         self.resize(600, 500)
 
         self.dataModel = DataFrameModel(data, parent=self)
-        self.dataModel.dataChanged.connect(self.save_and_close_enable)
         self.dataTable = DataFrameView(self, self.dataModel)
 
         self.layout.addWidget(self.dataTable)
@@ -728,29 +699,15 @@ class DataFrameEditor(QDialog):
         btn_layout.addWidget(self.bgcolor_global)
 
         btn_layout.addStretch()
-
-        self.btn_save_and_close = QPushButton(_('Save and Close'))
-        self.btn_save_and_close.setDisabled(True)
-        self.btn_save_and_close.clicked.connect(self.accept)
-        btn_layout.addWidget(self.btn_save_and_close)
-
-        self.btn_close = QPushButton(_('Close'))
-        self.btn_close.setAutoDefault(True)
-        self.btn_close.setDefault(True)
-        self.btn_close.clicked.connect(self.reject)
-        btn_layout.addWidget(self.btn_close)
+        bbox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        bbox.accepted.connect(self.accept)
+        bbox.rejected.connect(self.reject)
+        btn_layout.addWidget(bbox)
 
         self.layout.addLayout(btn_layout, 2, 0)
 
         return True
 
-    @Slot(QModelIndex, QModelIndex)
-    def save_and_close_enable(self, top_left, bottom_right):
-        """Handle the data change event to enable the save and close button."""
-        self.btn_save_and_close.setEnabled(True)
-        self.btn_save_and_close.setAutoDefault(True)
-        self.btn_save_and_close.setDefault(True)
-
     def change_bgcolor_enable(self, state):
         """
         This is implementet so column min/max is only active when bgcolor is
@@ -843,7 +800,9 @@ def test():
     out = test_edit(df1.iloc[0])
     assert_series_equal(result, out)
 
+    # Sorting large DataFrame takes time
     df1 = DataFrame(np.random.rand(100100, 10))
+    df1.sort(columns=[0, 1], inplace=True)
     out = test_edit(df1)
     assert_frame_equal(out, df1)
 
